#include "USART.h"



/****************************************************************
---------- USART INIT ------------------------------------------
****************************************************************/
void usart_init()
{

	// -------------------------
	// USART0
	// -------------------------
	
	uint16_t 	i = 0;
		
	// clear rx0 - buffer
	while (i<513)
		{
		rx0_buffer[i] = 0;
		i++;
		}
	// -------------------------
	
	// set TX0 PIN to output 
	PORTE = ~(1<< TX0);
	DDRE  =  (1<< TX0); 	
	
	// set status for tx reset
	usart_status.tx0 = 1;

	// set data format 8bit, 2 stopbit
	UCSR0C |= (1 << UCSZ00) | (1 << UCSZ01) | (1 << USBS0);

	// start usart0
	UCSR0B |= (1 << RXCIE0) | (1 << RXEN0) | (1 << UDRIE0);

	// set prescaler
	UBRR0L = BAUD_PRESCALE; 
	UBRR0H = (BAUD_PRESCALE >> 8); 

}







/****************************************************************
---------- RX0 INTERRUPT ROUTINE --------------------------------
****************************************************************/

ISR	(USART0_RX_vect)
{

	uint8_t		rx_data;
				
	// read data
	rx_data = UDR0;
		
	// check for data overrun
	if (UCSR0A & (1 << DOR0))
			{
			// clear DOR bit
			UCSR0A &= ~(1 << DOR0);
	
			rx0_cnt = 513;
			
			return;
			}
	
	
	
	// check for frame error
	if (UCSR0A & (1 << FE0))
			{
			// clear FE bit
			UCSR0A &= ~(1 << FE0);
			
			// reset channel counter
			rx0_cnt = 0;
						
			return;
			}
	
	
	
	// save data
	if ( rx0_cnt <= 512 )
			{
			rx0_buffer[rx0_cnt++] = rx_data;
			
			}

}





/****************************************************************
---------- TX0 INTERRUPT ROUTINE --------------------------------
****************************************************************/

ISR	(USART0_UDRE_vect)
{
			
	// requires	a break ?
	if (usart_status.tx0 == 1)
			{
			
			// generate reset cycle
			
			// deaktivate UDRIE0
			UCSR0B &= ~( (1 << UDRIE0) | ( 1<< TXEN0) );
			
			// clear TX PIN for RESET
			PORTE &= ~(1 << TX0);
			
			// clear tx_counter
			tx0_cnt = 0;
			
			// set preload value and prescaler for timer
			set_timer0(190, 3); 		
		
			return;
			}

		else
			{
			UDR0 = rx0_buffer[tx0_cnt++];
			}
			
                        if ( tx0_cnt >= 513 )
					{
					usart_status.tx0 = 1;
					}
			
			
}

/****************************************************************
---------- TIM0_OVR INTERRUPT ROUTINE ---------------------------
---------- GENERATE TIMING FOR DMX-------------------------------
****************************************************************/

ISR	(TIMER0_OVF_vect)
{
	
	// requires	a break ?
	if (usart_status.tx0 == 1)
			{
			usart_status.tx0 = 0;

			// set TX PIN for MaB
			PORTE |= (1 << TX0);
			
			// set preload value and prescaler for timer
			set_timer0(244, 1); 		
			return;
			}

		else
			{
			
			// activate Transmitter
			UCSR0B |= (1 << UDRIE0) | (1 << TXEN0);

			// stop timer
			set_timer0(0, 0); 		
							
			return;
			}

}



/****************************************************************
---------- SET TIMER0 FOR TXD RESET------------------------------
****************************************************************/

void set_timer0 (uint8_t preload, uint8_t prescale)
	{
	
	TCNT0 = preload;
	TCCR0 = prescale;
	TIMSK |= (1 << TOIE0);

	}
